<?php

require 'vendor/autoload.php';

function buyplaytix_scripts()
{
  $oauth = new \BuyPlayTix\Wordpress\OAuth();
  $producer = $oauth->getProducer();

  wp_enqueue_style('bpt-ecss.css', $producer->css_path . '/ecss.css');
  wp_enqueue_script('jquery.scrollTo', plugins_url('/buyplaytix/js/jquery.scrollTo-1.4.2-min.js'), array('jquery'));
  wp_enqueue_script('bpt-common', plugins_url('/buyplaytix/js/common.js'), array('jquery'));
  wp_enqueue_script('bpt-wordpress', plugins_url('/buyplaytix/js/wordpress.js'), array('jquery'));
  wp_enqueue_script('bpt-transaction-common', plugins_url('/buyplaytix/js/transaction-common.js'), array('jquery'));
  wp_enqueue_script('bpt-donate', plugins_url('/buyplaytix/js/donate.js'), array('jquery'));
  wp_enqueue_script('fullcalendar', plugins_url('/buyplaytix/js/vendor/fullcalendar/fullcalendar.min.js'), array('jquery'));
  wp_enqueue_script('fullcalendar.bpt', plugins_url('/buyplaytix/js/vendor/fullcalendar/bpt.js'), array('jquery'));
  wp_enqueue_style('fullcalendar.css', plugins_url('/buyplaytix/css/vendor/fullcalendar/fullcalendar.css'));
}

function buyplaytix_admin_scripts()
{
  $oauth = new \BuyPlayTix\Wordpress\OAuth();
  $producer = $oauth->getProducer();

//  wp_enqueue_style('bpt-ecss.css', $producer->css_path . '/ecss.css');
  wp_enqueue_script('jquery.scrollTo', plugins_url('/buyplaytix/js/jquery.scrollTo-1.4.2-min.js'), array('jquery'));
  wp_enqueue_script('bpt-common', plugins_url('/buyplaytix/js/common.js'), array('jquery'));
  wp_enqueue_script('bpt-wordpress', plugins_url('/buyplaytix/js/wordpress.js'), array('jquery'));
  wp_enqueue_script('bpt-transaction-common', plugins_url('/buyplaytix/js/transaction-common.js'), array('jquery'));
  wp_enqueue_script('bpt-donate', plugins_url('/buyplaytix/js/donate.js'), array('jquery'));
//  wp_enqueue_script('fullcalendar', plugins_url('/buyplaytix/js/vendor/fullcalendar/fullcalendar.min.js'), array('jquery'));
//  wp_enqueue_script('fullcalendar.bpt', plugins_url('/buyplaytix/js/vendor/fullcalendar/bpt.js'), array('jquery'));
//  wp_enqueue_style('fullcalendar.css', plugins_url('/buyplaytix/css/vendor/fullcalendar/fullcalendar.css'));
}

add_action('wp_enqueue_scripts', 'buyplaytix_scripts');
add_action('admin_enqueue_scripts', 'buyplaytix_admin_scripts');


function buyplaytix_widgets()
{
  register_widget('BuyPlayTix\Wordpress\Widget\DonateWidget');
  register_widget('BuyPlayTix\Wordpress\Widget\CalendarWidget');
  register_widget('BuyPlayTix\Wordpress\Widget\ProductionWidget');
}

function buyplaytix_auth($user, $username, $password)
{
  if (empty($username) || empty($password)) return;

  $client = new \GuzzleHttp\Client();
  $res = $client->request('POST', \BuyPlayTix\Wordpress\OAuth::$BASE_PATH . "/signin.service", [
    'form_params' => [
      'action' => 'signin',
      'username' => $username,
      'password' => $password,
    ]
  ]);
  $output = json_decode($res->getBody());
  if ($output->responseCode !== 200) {
    return new WP_Error( 'denied', __($output->responseObj) );
  }
  $email = $output->responseObj->email;
  $first_name = $output->responseObj->first_name;
  $last_name = $output->responseObj->last_name;
  $userobj = new WP_User();
  $user = $userobj->get_data_by( 'email', $email); // Does not return a WP_User object 🙁
  $user = new WP_User($user->ID); // Attempt to load up the user with that ID

  if( $user->ID == 0 ) {
    $userdata = [
      'user_email' => $email,
      'user_login' => $email,
      'first_name' => $first_name,
      'last_name' => $last_name,
    ];
    $defaultRole = get_option("buyplaytix_membership_none", NULL);
    if ($defaultRole) {
      $userdata['role'] = $defaultRole;
    }
    $new_user_id = wp_insert_user( $userdata ); // A new user has been created

    // Load the new user info
    $user = new WP_User ($new_user_id);
  }
  return $user;
}

function buyplaytix_membership($user, $username, $password)
{
  if (empty($user) || array_key_exists("errors", $user)) {
    return $user;
  }
  if ( in_array( 'admin', (array) $user->roles ) ) {
    // we're not messing with admin users
    return $user;
  }
  $roles = wp_roles();
  $oauth = new BuyPlayTix\Wordpress\OAuth();
  $subscriptions = $oauth->getSubscriptions($user);
  $currentRole = get_option("buyplaytix_membership_none", NULL);
  foreach($subscriptions as $subscription) {
    $fullKey = "buyplaytix_membership_" . $subscription->MEMBERSHIP->UID;
    $roleId = get_option($fullKey, NULL);
    $currentRole = $roleId;
  }
  if (!$currentRole) {
    return $user;
  }
  if ( !in_array( $currentRole, (array) $user->roles ) ) {
    // add this role
    $user->role = $currentRole;
    $new_user_id = wp_update_user( $user ); // A new user has been created
    if ( is_wp_error( $new_user_id ) ) {
      return $new_user_id;
    }
  }
  return $user;
}

add_action('widgets_init', 'buyplaytix_widgets');
add_action('admin_init', array('BuyPlayTix\Wordpress\Plugin', 'admin_init'));
add_action('admin_menu', array('BuyPlayTix\Wordpress\Plugin', 'admin_menu'));
add_action('admin_notices', array('BuyPlayTix\Wordpress\Plugin', 'admin_notices'));
add_action('add_meta_boxes', array('BuyPlayTix\WordPress\Plugin', 'register_production_box'));
add_action('save_post', array('BuyPlayTix\WordPress\Plugin', 'production_box_save'));

add_filter('plugin_action_links', array('BuyPlayTix\WordPress\Plugin', 'settings_link'), 10, 2);
add_filter('authenticate', 'buyplaytix_auth', 10, 3);
add_filter('authenticate', 'buyplaytix_membership', 10, 3);

add_shortcode('bpt_calendar', array('BuyPlayTix\Wordpress\Plugin', 'calendar_shorttag'));
add_shortcode('bpt_tickets', array('BuyPlayTix\Wordpress\Plugin', 'tickets_shorttag'));
add_shortcode('bpt_donate', array('BuyPlayTix\Wordpress\Plugin', 'donate_shorttag'));
add_shortcode('bpt_minical', array('BuyPlayTix\Wordpress\Plugin', 'minical_shorttag'));
add_shortcode('bpt_when', array('BuyPlayTix\Wordpress\Plugin', 'when_shorttag'));
add_shortcode('bpt_logo', array('BuyPlayTix\Wordpress\Plugin', 'logo_shorttag'));
add_shortcode('bpt_additional_info', array('BuyPlayTix\Wordpress\Plugin', 'additional_info_shorttag'));
add_shortcode('bpt_location', array('BuyPlayTix\Wordpress\Plugin', 'location_shorttag'));
add_shortcode('bpt_history', array('BuyPlayTix\Wordpress\Plugin', 'history_shorttag'));







